// CTomPoiEdr.hpp - POI Editor for TomTom Navigation
////////////////////////////////////////////////////////////////////////////////////////////////

// Marks the file we have included
#ifndef _CTOMPOIEDR_INCLUDED_
#define _CTOMPOIEDR_INCLUDED_

// Framework header file should be included first
#ifndef _WINAFX_INCLUDED_
#error  CTomPoiEdr - Framework header file <WinAfx.hpp> should be included first.
#endif

// Included only once by the compiler in a build
#pragma once

////////////////////////////////////////////////////////////////////////////////////////////////

#include "Resource.hpp"
#include "WinMain.hpp"
#include "CCheckDataDlg.hpp"
#include "CCheckPoiDlg.hpp"
#include "CCombColDlg.hpp"
#include "CConvertFormatDlg.hpp"
#include "CCustTbDlg.hpp"
#include "CFileDlg.hpp"
#include "CFindReplDlg.hpp"
#include "CFontDlg.hpp"
#include "CIEPane.hpp"
#include "CLvBkDlg.hpp"
#include "CLvCombo.hpp"
#include "CLvCtrl.hpp"
#include "CLvEdit.hpp"
#include "CLvHeader.hpp"
#include "COpenFavDlg.hpp"
#include "COptionDlg.hpp"
#include "CSbCtrl.hpp"
#include "CTbCtrl.hpp"
#include "CTranNameDlg.hpp"

// GDI+ Image class
using namespace Gdiplus;
typedef Image*          PIMAGE;
typedef ImageCodecInfo* PICI;

// GPS dump device (gpsd)
enum GPSDUMP
{
	GPSD_NONE = 0,
	GPSD_V3,
	GPSD_V4,
};

// Acceptable POI files in GPS device
enum
{
	GPSF_FIRST = 0,
	GPSF_OV2   = GPSF_FIRST,
	GPSF_BMP,
	GPSF_OGG,
	GPSF_RPF,
	GPSF_VIF,
	GPSF_COUNT,
};

// File item for GPS device (gpsf)
typedef struct GPSFILE
{
	PTSTR    pszName;
	PTSTR    pszAttr;
	GPSFILE* pNext;
} * PGPSFILE, ** PPGPSFILE;

// File list for GPS device (gpsfl)
typedef struct GPSFILELIST
{
	PTSTR    pszHtmData;
	PGPSFILE pGPSF[GPSF_COUNT];
} * PGPSFILELIST;

// Thread parameters for GPS device, DELETE verb (gpsd)
typedef struct GPSDELETE
{
	HINTERNET    hInternet;
	PCTSTR       pszName;
	PGPSFILELIST pGPSFL;
} * PGPSDELETE;

// Thread parameters for GPS device, POST verb (gpsp)
typedef struct GPSPOST
{
	HINTERNET hInternet;
	PCTSTR    pszName;
	PCTSTR    pszFileName;
	UINT      nSaveFlags;
} * PGPSPOST;

class CTomPoiEdr : public CFrameWnd
{
public:
#pragma pack(1)
	// Deleted POI record (type FRT_DELETED)
	typedef struct POIRECORD0
	{
		BYTE  byType;  // Type, always FRT_DELETED
		DWORD cbSize;  // Size of this record, in bytes
	} * PPOIRECORD0;

	// Skipper POI record (type FRT_SKIPPER)
	typedef struct POIRECORD1
	{
		BYTE  byType;  // Type, always FRT_SKIPPER
		DWORD cbSize;  // Size of enclosed rectangle, including this record, in bytes
		LONG  lLngW;   // Longitude coordinate of the west edge of the rectangle
		LONG  lLatS;   // Latitude coordinate of the south edge of the rectangle
		LONG  lLngE;   // Longitude coordinate of the east edge of the rectangle
		LONG  lLatN;   // Latitude coordinate of the north edge of the rectangle
	} * PPOIRECORD1;

	// Simple POI record (type FRT_SIMPLE)
	typedef struct POIRECORD2
	{
		BYTE  byType;     // Type, always FRT_SIMPLE
		DWORD cbSize;     // Size of this record, in bytes
		LONG  lLng;       // Longitude coordinate of POI
		LONG  lLat;       // Latitude coordinate of POI
		char  szName[1];  // Name of POI (Size = cbSize - m - 13)
		/*
		char  szMemo[m]   -> Memo of POI (Unicode string)
		*/
	} * PPOIRECORD2;

	// Extended of simple POI record (type FRT_EXSIMPLE)
	typedef struct POIRECORD3
	{
		BYTE  byType;     // Type, always FRT_EXSIMPLE
		DWORD cbSize;     // Size of this record, in bytes
		LONG  lLng;       // Longitude coordinate of POI
		LONG  lLat;       // Latitude coordinate of POI
		char  szName[1];  // Name of POI (Size = cbSize - p - e - 13)
		/*
		char  szPoiId[p]  -> Identifier of POI
		char  szExtra[e]  -> Extra data (not used yet)
		*/
	} * PPOIRECORD3;

	// Decoded table for record 9 (pr9dt)
	typedef struct POIRECORD9DT
	{
		PCSTR pszCode;  // String of character code
		int   cbCode;   // String length of character code
		WCHAR wChar;    // Decoded character
	} const CPOIRECORD9DT;

	// Variety of simple POI record (type FRT_VARSIMPLE)
	typedef struct POIRECORDF
	{
		BYTE  byType;     // Type, always FRT_VARSIMPLE
		DWORD cbSize;     // Size of this record, in bytes
		LONG  lLng;       // Longitude coordinate of POI
		LONG  lLat;       // Latitude coordinate of POI
		char  szName[1];  // Name of POI (Size = cbSize - 13)
	} * PPOIRECORDF;

	// Simple POI record (type FRT_ENSIMPLE)
	typedef struct POIRECORD17
	{
		BYTE byType;     // Type, always FRT_ENSIMPLE
		BYTE cbSize;     // Size of this record = cbSize + 8, in bytes
		BYTE byLng[3];   // Longitude coordinate of POI
		BYTE byLat[3];   // Latitude coordinate of POI
		char szName[1];  // Name of POI (Size = cbSize)
	} * PPOIRECORD17;

	// Rich POI File record (type FRT_RPF)
	typedef struct POIRECORD1D
	{
		BYTE byType;         // Type, always FRT_RPF
		BYTE cbSize;         // Size of this record = cbSize + 11, in bytes
		LONG lLng;           // Longitude coordinate of POI
		LONG lLat;           // Latitude coordinate of POI
		BYTE byNET;          // Name encoding type
		BYTE byInfoId[3];    // Identifier of POI info
		/*
		byInfoId is combined with the following elements
		----------------------
		BYTE byIconId;       -> Identifier of POI icon
		BYTE byDetailId[3];  -> Identifier of POI detail
		----------------------

		The following types have no byIconId element
			NET_ANSI
			NET_TOMTOM_6
			NET_UNICODE

		The following type has no byDetailId element
			NET_TOMTOM_5
		*/
		char szName[1];      // Name of POI (Size = cbSize - 3)
	} * PPOIRECORD1D;

	// Safety camera record (type FRT_SC)
	typedef struct POIRECORD1E
	{
		BYTE byType;    // Type, always FRT_SC
		BYTE byCamera;  // Camera mode
		LONG lLng;      // Longitude coordinate of SC
		LONG lLat;      // Latitude coordinate of SC
		BYTE bySpeed;   // Speed limit
	} * PPOIRECORD1E;

	// Variety of Rich POI File record (type FRT_VARRPF)
	typedef struct POIRECORD1F
	{
		BYTE  byType;         // Type, always FRT_VARRPF
		DWORD cbSize;         // Size of this record, in bytes
		LONG  lLng;           // Longitude coordinate of POI
		LONG  lLat;           // Latitude coordinate of POI
		BYTE  byIconId;       // Identifier of POI icon
		BYTE  byDetailId[3];  // Identifier of POI detail
		char  szName[1];      // Name of POI (Size = cbSize - 17)
	} * PPOIRECORD1F;

	// Signal mark record (type FRT_SIGNAL)
	typedef struct POIRECORD64
	{
		BYTE  byType;       // Type, always FRT_SIGNAL
		DWORD cbSize;       // Size of this record, in bytes
		BYTE  bySignal[9];  // Signal mark of POI file
		BYTE  byAddDelta;   // Addition delta for signal mark
		BYTE  byPad1;       // Pad byte, always 0
		BYTE  bySubDelta;   // Subtraction delta for signal mark
		BYTE  byPad2[4];    // Pad bytes, always 0
	} * PPOIRECORD64;
	#define SPOIRECORD64 static POIRECORD64

	// Favorite record
	typedef struct FAVORITERECORD
	{
		BYTE bySignal[6];    // Signal mark (0x04 0x00 0x?? 0x00 0x00 0x00)
		                     // Third byte of signal mark is meaning:
		                     //   0x01 - Point on map or by coordinates
		                     //   0x03 - Favorite
		                     //   0x04 - Home
		                     //   0x05 - Address
		                     //   0x06 - Point of Interest
		                     //   0x07 - Position of last stop
		WORD cbSizeOC;       // Size of original coordinates, always 8
		LONG lLngOC;         // Longitude (original)
		LONG lLatOC;         // Latitude (original)
		WORD cbSizeRC;       // Size of refixed coordinates, always 8
		LONG lLngRC;         // Longitude (refixed)
		LONG lLatRC;         // Latitude (refixed)
		char szName[1];      // Name of favorite, collection of the following elements
		/*
		BYTE cbTownship      -> Size of township (cbTownship = t + 27)
		char szTownship[t]   -> Township string (t = cbTownship - 27)
		BYTE cbDoorplate     -> Size of doorplate (cbDoorplate = d + 27)
		char szDoorplate[d]  -> Doorplate string
		BYTE cbOfficial      -> Size of official (cbOfficial = o + 27)
		char szOfficial[o]   -> Official string
		*/
	} * PFAVORITERECORD;

	// GPI file header (gpih)
	typedef struct GPIHEADER
	{
		DWORD dwDummy1;     // Unknown
		DWORD dwDummy2;     // Unknown
		char  szGrmRec[8];  // Signal mark, always "GRMREC00"
		LONG  lCreation;    // Creation date and time (time_t)
		WORD  wDummy1;      // Unknown
		WORD  cbName;       // Size of file name, should be 6
		char  szName[6];    // File name, should be "my.gpi"
		DWORD dwDummy3;     // Unknown
		DWORD dwDummy4;     // Unknown
		char  szPOI[3];     // String literal, always "POI"
		char  cDummy[3];    // Unknown
		WORD  wDummy2;      // Unknown
		DWORD dwCodePage;   // Identifier of code page, for example: 950
		DWORD dwDummy5;     // Unknown
		DWORD dwDummy6;     // Unknown
		DWORD cbOffset;     // Offset to some info, in bytes (may not have)
		DWORD dwDummy7;     // Unknown
		char  szEN[2];      // String literal, always "EN"
		WORD  cbTitle;      // Size of file title, in bytes
		char  szTitle[1];   // File title
		/*
		DWORD dwDummy;      -> Unknown
		DWORD cbOffset;     -> Offset to some info, in bytes
		DWORD dwDummy;      -> Unknown
		LONG  lLat1;        -> Unknown coordinates 1
		LONG  lLng1;
		LONG  lLat2;        -> Unknown coordinates 2
		LONG  lLng2;
		---------------------> Begin: may not have the following elements
		char  cDummy[19];   -> Unknown
		LONG  lLat3;        -> Unknown coordinates 3
		LONG  lLng3;
		LONG  lLat4;        -> Unknown coordinates 4
		LONG  lLng4;
		char  cDummy[19];   -> Unknown
		LONG  lLat5;        -> Unknown coordinates 5
		LONG  lLng5;
		LONG  lLat6;        -> Unknown coordinates 6
		LONG  lLng6;
		---------------------> End: may not have the previous elements
		char  cDummy[7];    -> Unknown
		*/
	} * PGPIHEADER;

	// GPI data record (gpir)
	typedef struct GPIRECORD
	{
		DWORD dwSignal;       // Signal mark (0x02, 0x00, 0x08, 0x00)
		DWORD dwNextRec;      // Offset to next record, in bytes
		                      // Loc of next = loc of this record + dwNextRec + 12
		DWORD dwDummy1;       // Unknown
		LONG  lLat;           // Latitude coordinate of POI
		                      // Lat = lLat / 2^31 * 180
		LONG  lLng;           // Longitude coordinate of POI
		                      // Lng = lLng / 2^31 * 180
		char  cDummy[3];      // Unknown
		DWORD dwDummy2;       // Unknown
		char  szEN[2];        // String literal, always "EN"
		WORD  cbName;         // Size of POI name, in bytes
		char  szName[1];      // POI name
		/*
		char  cDummy[?];      -> Unknown
		-----------------------> Begin: may not have the following elements
		char  szEN[2];        -> String literal, always "EN"
		WORD  cbDesc;         -> Total size of POI description, in bytes
		char  szDesc1[?];     -> POI description 1
		char  szSep1[?];      -> Separater 1 (0x0D, 0x0A, 0x0D, 0x0A, ...)
		char  szDesc2[?];     -> POI description 2
		char  szSep2[?];      -> Separater 2
		...
		char  szDescN[?];     -> POI description N
		char  szSepN[?];      -> Separater N
		char  cOther[?];      -> Other elements
		-----------------------> End: may not have the previous elements
		*/
	} * PGPIRECORD;
#pragma pack()

	// Operates POI file (opf)
	enum OPERPOIFILE
	{
		OPF_READ = 0,
		OPF_WRITE,
		OPF_IMPORT,
		OPF_EXPORT,
		OPF_WE_MASK = 0x1,  // OPF_WRITE or OPF_EXPORT
	};

	// File type for reading (FILETYPEREAD, ftr)
	enum
	{
		FTR_CFG = 0,
		FTR_OV2,
		FTR_CPB,
		FTR_KML,
		FTR_GPX,
		FTR_GPI,
		FTR_CSV,
		FTR_ASC,
		FTR_POI,
		FTR_TXT,
		FTR_CSVA,
		FTR_CSVU,
		FTR_CSVU8,
		FTR_ASCA,
		FTR_ASCU,
		FTR_ASCU8,
		FTR_POIA,
		FTR_POIU,
		FTR_POIU8,
		FTR_TXTA,
		FTR_TXTU,
		FTR_TXTU8,
	};

	// File type for writing (FILETYPEWRITE, ftw)
	enum
	{
		FTW_OV2   = FTR_OV2,
		FTW_CPB   = FTR_CPB,
		FTW_KML   = FTR_KML,
		FTW_GPX   = FTR_GPX,
		FTW_GPI   = FTR_GPI,
		FTW_CSV   = FTR_CSV,
		FTW_ASC   = FTR_ASC,
		FTW_POI   = FTR_POI,
		FTW_TXT   = FTR_TXT,
		FTW_CSVA  = FTR_CSVA,
		FTW_CSVU  = FTR_CSVU,
		FTW_CSVU8 = FTR_CSVU8,
		FTW_ASCA  = FTR_ASCA,
		FTW_ASCU  = FTR_ASCU,
		FTW_ASCU8 = FTR_ASCU8,
		FTW_POIA  = FTR_POIA,
		FTW_POIU  = FTR_POIU,
		FTW_POIU8 = FTR_POIU8,
		FTW_TXTA  = FTR_TXTA,
		FTW_TXTU  = FTR_TXTU,
		FTW_TXTU8 = FTR_TXTU8,
	};

	// File structure type (fst)
	enum FILESTRUCTTYPE
	{
		FST_GENERAL = 0,  // General file or Rich POI File
		FST_SC,           // Safety camera
		FST_CPB,          // Cell phonebook
	};

	// Record type of POI file (FILERECORDTYPE, frt)
	enum
	{
		FRT_DELETED   = 0x00,  // Deleted
		FRT_SKIPPER   = 0x01,  // Skipper
		FRT_SIMPLE    = 0x02,  // Simple
		FRT_EXSIMPLE  = 0x03,  // Extended of simple
		FRT_VARSIMPLE = 0x0F,  // Variety of simple
		FRT_ENSIMPLE  = 0x17,  // Encoded of simple
		FRT_RPF       = 0x1D,  // Rich POI File
		FRT_SC        = 0x1E,  // Safety camera
		FRT_VARRPF    = 0x1F,  // Variety of Rich POI File
		FRT_SIGNAL    = 0x64,  // Signal mark
	};

	// Token in POI files (FILETOKEN, ft)
	enum
	{
		FT_TOMTOM    = 0x3E3E,  // TomTom character set
		FT_UTF8H     = 0xBF,    // High order byte of UTF-8 mark
		FT_UTF8L     = 0xBBEF,  // Low order word of UTF-8 mark
		FT_UTF8S2    = 2,       // Size of UTF-8 character (2 bytes)
		FT_UTF8S3    = 3,       // Size of UTF-8 character (3 bytes)
		FT_UTF8S3FC  = 0xE0,    // First character (gl) of 3 bytes of UTF-8
		FT_ASCII     = 0x21,    // Ascii mark in high byte for TomTom
		FT_POI_SEP   = 0x3E,    // Separator between name and telephone columns in POI
		FT_ITN_SEP   = 0x7C,    // Separator between columns in ITN
		FT_SCM_FIRST = 0x81,                  // First mode of safety camera
		FT_SCM_FIXED = FT_SCM_FIRST,          // Fixed
		FT_SCM_MOBILE,                        // Mobile
		FT_SCM_TRAFFIC_LIGHT,                 // Traffic light
		FT_SCM_TOLL_ROAD,                     // Toll road
		FT_SCM_VEHICLE_RESTRICT,              // Vehicle restriction
		FT_SCM_OTHER,                         // Other
		FT_SCM_MOBILE_2,                      // Another mobile
		FT_SCM_AVERAGE_START,                 // Average speed check (Start)
		FT_SCM_AVERAGE_END,                   // Average speed check (End)
		FT_SCM_BIDIRECT_CAMERA,               // Bidirectional camera
		FT_SCM_ACCIDENT_BLACKSPOT,            // Accident blackspot location
		FT_SCM_GRADE_CROSSING,                // Grade crossing without gates
		FT_SCM_LAST = FT_SCM_GRADE_CROSSING,  // Last mode of safety camera
		FT_RPF_UEM  = 0x5B,  // Encoding mark in RPF (Unicode)
		FT_RPF_UCS  = 0xFF,  // Character separator in RPF (Unicode)
	};

	// Encoding type of POI name for record 9 (NAMEENCODETYPE, net)
	enum
	{
		NET_ANSI     = 0x02,  // ANSI character set, byInfoId is 3 bytes
		NET_EXANSI   = 0x03,  // Extended of ANSI character set, byInfoId is 4 bytes
		NET_TOMTOM_5 = 0x05,  // TomTom character set, byInfoId is 1 byte
		NET_TOMTOM_6 = 0x06,  // TomTom character set, byInfoId is 3 bytes
		NET_TOMTOM_7 = 0x07,  // TomTom character set, byInfoId is 4 bytes
		NET_UNICODE  = 0x0A,  // Unicode character set, byInfoId is 3 bytes
	};

	// POI coordinate block (pcb)
	typedef struct POICOORDBLOCK
	{
		POIRECORD1 pr1;        // Skipper POI record
		LONG       lLng;       // Longitude coordinate of coordinate block
		LONG       lLat;       // Latitude coordinate of coordinate block
		PINT       piLvItem;   // Items in coordinate block
		int        ceLvItem;   // Number of items
		BOOL       bCombined;  // Coordinate block is combined with another
	} * PPOICOORDBLOCK;
	#define SCPOICOORDBLOCK static const POICOORDBLOCK

	// Combine coordinate block (ccb)
	typedef struct COMBINECOORDBLOCK
	{
		POIRECORD1 pr1;    // Skipper POI record
		PINT       piPCB;  // Combined coordinate blocks
		int        cePCB;  // Number of coordinate blocks
	} * PCOMBINECOORDBLOCK;

	// For pcb and ccb
	enum
	{
		m_ecxCB      = 5000,        // Width of coordinate block
		m_ecyCB      = m_ecxCB,     // Height of coordinate block
		m_eceMinBI   = 3,           // Minimum number of items in coordinate block
		m_ecxAB      = 10000,       // Width of adjacent block
		m_ecyAB      = m_ecxAB,     // Height of adjacent block
		m_eceMinAB   = m_eceMinBI,  // Minimum number of adjacent blocks
		m_eceAllocCB = 32,          // Number of allocating coordinate blocks
		m_eceAllocBI = 32,          // Number of allocating items in coordinate block
	};

	// Clipboard data (cbd)
	typedef struct CBD
	{
		// Length of individual data, including null-terminated character
		int ccData[LVC_COUNT - 1];

		// Data of POI or ITN (null-terminated strings)
		WCHAR szwData[1];
	} * PCBD;
	enum
	{
		// Size of CBD structure
		m_ecbCBD = memberof(CBD, szwData),

		// Minimum number, in bytes, for allocating memory (Copy/Cut POI)
		m_ecbMinAllocCP = e_eccAllCol * sizeof(WCHAR) * 1000,
	};

	// Copy Coordinates
	enum
	{
		m_eccAllocCC = 64,  // Number of allocating memory for clipboard
	};

	// Menu help
	struct MENUHELP
	{
		HMENU hMenu;
		UINT  nBaseId;
	};

	// ARGB colors for toolbar hot items (argb)
	typedef struct ARGB
	{
		BYTE byRed;
		BYTE byGreen;
		BYTE byBlue;
		BYTE byAlpha;
	} * PARGB;

	enum
	{
		// Unicode byte-order mark (BOM)
		m_eUBOM = 0xFEFF,

		// Signal mark
		m_eMaxSm         = MAX_BYTE,  // Maximum signal mark
		m_eMinSm         = 1,         // Minimum signal mark
		m_eSmChkSum      = 0x187,     // Check sum
		m_eSmChkSumSC    = 0x87,      // Check sum for safety camera
		m_eSmAddDelta    = 0x80,      // Addition delta
		m_eMaxSmSubDelta = m_eMaxSm,  // Maximum subtraction delta
		m_eMinSmSubDelta = m_eMinSm,  // Minimum subtraction delta
		m_eMaxSmSum      = m_eSmChkSum - m_eSmAddDelta + m_eMaxSmSubDelta,  // Maximum sum
		m_eMinSmSum      = m_eSmChkSum - m_eSmAddDelta + m_eMinSmSubDelta,  // Minimum sum
		m_eceSmSum       = 4,         // Number of elements for sum

		// Converts characters
		m_eVarDelta   = 0xA5,    // Delta for variety of simple POI record
		m_eMapDelta   = 0x1EEC,  // Delta for mapping TomTom characters
		m_ePageDelta  = 0xD5,    // Delta for code page mapping
		m_eCharAdjD   = 0x23,    // Character mapping adjust in duplicated part
		m_eDupDelta   = 0x29,    // Delta for duplicated part
		m_eCharAdjL   = 0x25,    // Character mapping adjust in low part
		m_eCharAdjH   = 0x5C,    // Character mapping adjust in high part
		m_eccCharPage = 256,     // Number of characters in a page
		m_eccStdAnsi  = 256,     // Number of standard ASNI characters
		m_eAsciiPage  = 0x2100,  // Ascii page in TomTom
		m_eLatinPage  = 0x2600,  // Latin page in TomTom
		m_eCJKRPage   = 0x5D00,  // CJK radical page in TomTom
		m_eCJKCPage   = 0xF000,  // CJK compatibility page in TomTom
		m_eCJKCDelta  = 0x5000,  // Delta for CJK compatibility part

		// Encoded coordinates
		m_eEnCoordSubDelta  = 80 * e_eCoordPump,    // Subtraction
		m_eEnCoordAdjDelta  = -180 * e_eCoordPump,  // Adjust
		m_eEnCoordRollDelta = 360 * e_eCoordPump,   // Roll

		// RPF record
		m_eRpfRdDelta   = 11,                          // Delta for record size
		m_eRpfNameDelta = 3,                           // Delta for size of name
		m_ecbMaxRpfName = MAX_BYTE - m_eRpfNameDelta,  // Maximum number, in bytes, of name
		m_eceMaxRpfId   = 0xFFFFFF,                    // Maximum number of indentifier
		m_eRpfDefIconId = 1,                           // Default icon indentifier

		// Speed limit
		e_eMinSpeed  = 20,
		e_eMaxSpeed  = 140,
		e_eDefSpeed  = 60,
		e_eNonSpeed  = 0,
		e_eDivSpeed  = 10,
		e_eBaseSpeed = 5,

		// Favorite record
		m_eFavNameDelta = 27,  // Delta for size of name
		m_ecbMinFavRd   = 30,  // Minimum number, in bytes, of a record
		m_eceMaxFavPoi  = 96,  // Maximum number of POIs can be accessible by GPS device

		// TomFile/WinFile
		m_eceNumber = _T('9') - _T('0') + 1,
		m_eceAlpha  = _T('Z') - _T('A') + 1,

		// Menu background color
		m_ecrMenuBkgnd = RGB(238, 238, 234),

		// Menu item
		m_eFileMenuPos  = 0,
		m_eRecentInsPos = 13,
		m_ecxRecentItem = 300,
		m_eToolsMenuPos = 3,
		m_eLastMenuPos  = 4,
		m_eceMenuHelp   = 2,

		// Menu item bitmaps
		m_ecxMenuBmpPadL = 2,
		m_ecyMenuBmpPadT = m_ecxMenuBmpPadL,
		m_ecxMenuBmpPadR = m_ecxMenuBmpPadL,
		m_ecyMenuBmpPadB = m_ecyMenuBmpPadT,
		m_ecyMenuBmpPadG = 5,

		// Popup menu item
		m_eTbPopMenu = 0,
		m_eHdPopMenu,
		m_eLvPopMenu,

		// Minimum, maximum and default number of recent files
		m_eceMinRecentFile = 1,
		m_eceMaxRecentFile = 9,
		m_eceDefRecentFile = 7,

		// Minimum, maximum and default number of minutes to backup file
		m_eceMinBackupIntl = 1,
		m_eceMaxBackupIntl = 99,
		m_eceDefBackupIntl = 10,

		// Index of Google Maps language
		m_eMinGmLang = 0,
		m_eMaxGmLang = IDS_GM_LANG_NAME8 - IDS_GM_LANG_NAME0,
		m_eceGmLang  = m_eMaxGmLang + 1,
		m_eDefGmLang = 0,

		// Toolbar border, divider and background colors
		m_ecrTbBorder    = RGB(182, 188, 204),
		m_ecrTbBorderXP  = RGB(172, 168, 153),
		m_ecrTbDivider   = RGB(160, 160, 160),
		m_ecrTbDividerXP = m_ecrTbBorderXP,
		m_ecrTbBkgnd     = RGB(224, 248, 255),
		m_ecrTbBkgndXP   = RGB(242, 242, 238),

		// Toolbar styles
		m_eTbStyle = WS_CHILD | WS_CLIPSIBLINGS |
			CCS_ADJUSTABLE | CCS_NODIVIDER | CCS_NORESIZE |
			TBSTYLE_CUSTOMERASE | TBSTYLE_FLAT | TBSTYLE_TOOLTIPS,

		// Toolbar extended styles
		m_eTbExStyle = TBSTYLE_EX_DOUBLEBUFFER | TBSTYLE_EX_HIDECLIPPEDBUTTONS,

		// Padding for toolbar control
		m_eyTbTop   = -2,
		m_eyTbTopXP = 0,
		m_ecxTbPad  = 3,
		m_ecyTbPad  = m_ecxTbPad,
		m_ecyTbDiv  = 2,
		m_eyTbPad   = m_ecyTbDiv + m_ecyTbPad,

		// Toolbar chevron
		m_ecxTbChevron      = 8,
		m_ecxTbChevronImage = 8,
		m_ecyTbChevronImage = 5,
		m_eTbChevronImageL  = 0,
		m_eTbChevronImageS,

		// Bitmap index of toolbar button
		m_eFileNew = 0,
		m_eFileOpen,
		m_eFileOpenFav,
		m_eFileOpenDevice,
		m_eFileDumpDevice,
		m_eFileSave,
		m_eFileSaveAs,
		m_eFileSaveDevice,
		m_eFileImport,
		m_eFileExport,
		m_eFileQuery,
		m_eFileTranslate,
		m_eFileConvert,
		m_ePoiUndo,
		m_ePoiRedo,
		m_ePoiAdd,
		m_ePoiInsert,
		m_ePoiCombineCol,
		m_ePoiEdit,
		m_ePoiDelete,
		m_ePoiClearCol,
		m_ePoiCut,
		m_ePoiCopy,
		m_ePoiPaste,
		m_ePoiQueryCoord,
		m_ePoiQueryAddr,
		m_ePoiFind,
		m_ePoiFindNext,
		m_ePoiReplace,
		m_ePoiReplaceNext,
		m_ePoiMessCoord,
		m_ePoiCheck,
		m_ePoiAddFav,
		m_ePoiAddItn,
		m_ePosUp,
		m_ePosDown,
		m_ePosSetBP,
		m_ePosMyBP,
		m_ePosSortAscend,
		m_ePosSortDescend,
		m_ePosGoogleMap,
		m_ePosGoogleEarth,
		m_eToolsQGPSfix,
		m_eToolsLvFont,
		m_eToolsLvBkgnd,
		m_eToolsOption,
		m_eHelpTomHome,
		m_eHelpTomUsers,
		m_eHelpTpeHome,
		m_eHelpMailto,
		m_eHelpAbout,
		m_eceTbImage,

		// Toolbar image and button
		m_ecxTbImageL = 24,
		m_ecyTbImageL = 24,
		m_ecxTbImageS = 16,
		m_ecyTbImageS = 16,

		// Header styles
		m_eHdStyle = WS_CHILD | WS_CLIPSIBLINGS | WS_VISIBLE | HDS_BUTTONS | HDS_HOTTRACK,

		// Padding for header control
		m_eyHdPad       = m_ecyTbDiv,
		m_ecyHdPad      = 5,
		m_ecxHdColPad   = 0,
		m_ecxHdColPadXP = 2,

		// List view styles
		m_eLvStyle = WS_CHILD | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | WS_VISIBLE |
			LVS_NOCOLUMNHEADER | LVS_REPORT,

		// List view extended styles
		m_eLvExStyle   = LVS_EX_DOUBLEBUFFER | LVS_EX_FULLROWSELECT | LVS_EX_LABELTIP,
		m_eLvExStyleXP = m_eLvExStyle | LVS_EX_GRIDLINES,

		// Column padding image for list view
		m_ecxLvColPadIL = 1,

		// Column widths of list view
		m_ecxLvNameCol = 200,
		m_ecxLvLngCol  = 100,
		m_ecxLvLatCol  = 100,
		m_ecxLvTelCol  = 150,
		m_ecxLvMemoCol = 150,
		m_ecxLvDistCol = 100,

		// Padding and adjusting for list view
		m_exLvNameAdj    = 4,
		m_exLvDistAdjL   = 6,
		m_exLvDistAdjR   = 8,
		m_exLvDistAdjLXP = 6,
		m_exLvDistAdjRXP = 6,

		// Edit styles
		m_eEdStyle = WS_BORDER | WS_CHILD | WS_CLIPSIBLINGS | ES_AUTOHSCROLL,

		// Padding and adjusting for edit control
		m_exEdPad  = 1,
		m_eyEdPad  = 2,
		m_exEdAdjL = 4,
		m_exEdAdjR = m_ecxLvLngCol,

		// Combo box styles
		m_eCbStyle = WS_BORDER | WS_CHILD | WS_CLIPSIBLINGS | WS_VSCROLL | CBS_DROPDOWNLIST,

		// List box height of combo box
		m_ecyCbLb = 300,

		// Status bar styles
		m_eSbStyle = WS_CHILD | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | WS_VISIBLE | CCS_BOTTOM,

		// Status bar parts
		m_eInfoPart = 0,
		m_eBmpPart,
		m_ePoiPart,
		m_eTimePart,
		m_eceSbPart,
		m_ecyMaxBmp = 24,

		// Progress styles
		m_ePbStyle   = WS_CHILD | WS_VISIBLE,
		m_ePbMqStyle = m_ePbStyle | PBS_MARQUEE,

		// Progress marquee animation
		m_ePbMqIntl = 20,

		// Padding for progress control
		m_ecxPbPadL = 12,
		m_ecxPbPadR = 8,
		m_ecyPbPadT = 4,
		m_ecyPbPadB = 6,

		// Timer identifiers
		m_eMainTimerId = 7,
		m_eIEPaneTimerId,
		m_eGmSelTimerId,
		m_eBackupTimerId,
		m_eExitTimerId,

		// Timer interval
		m_eMainTimerIntl  = 500,
		m_eGmSelTimerIntl = 750,
		m_eExitTimerIntl  = 2000,
		m_eMinuteIntl     = 60000,

		// New version notification
		m_eNewVerIntl    = 604800,  // Notification interval, in seconds
		m_eNewVerTimeout = 10000,   // Timeout value for Notify Icon

		// IE pane and separator
		m_ecxMinIEPane       = 100,
		m_ecxIEPaneSep       = 6,
		m_ecrIEPaneSep       = RGB(196, 218, 250),
		m_ecrIEPaneSepHilite = RGB(240, 240, 96),
		m_ecrIEPaneSepBorder = RGB(127, 157, 185),
		m_eIEPaneTimerIntl   = 250,

		// Google Maps
		m_ecbGmBuffer   = 64 * 1024,
		m_eGmQuerySleep = 500,
		m_ecbGmHtmDoc   = 1024 * 1024,

		// Google Maps Get commands
		m_eGmGetAdd = 0,
		m_eGmGetInsert,
		m_eGmGetUpdate,
		m_eGmGetSelect,
		m_eGmGetDelete,
		m_eGmGetEdit,
		m_eceGmGet,

		// Google Maps record items
		m_eGmTokenGet = CHR_LT,
		m_eGmTokenSep = CHR_TAB,
		m_eGmRdName   = LVC_NAME,
		m_eGmRdLng,
		m_eGmRdLat,
		m_eGmRdTel,
		m_eGmRdMemo,
		m_eGmRdNewLng,
		m_eGmRdNewLat,
		m_eGmRdNewMemo,
		m_eceGmRdItem,

		// Google Maps Put commands
		m_eGmPutSelect = 0,
		m_eGmPutDelete,
		m_eGmPutUpdate,
		m_eGmPutLFT,  // e_bLngFirst   = true
		m_eGmPutLFF,  // e_bLngFirst   = false
		m_eGmPutDFT,  // e_bDmsFormat  = true
		m_eGmPutDFF,  // e_bDmsFormat  = false
		m_eGmPutCDT,  // e_bCompassDir = true
		m_eGmPutCDF,  // e_bCompassDir = false
		m_eGmPutMUT,  // e_bMetricUnit = true
		m_eGmPutMUF,  // e_bMetricUnit = false
		m_eGmPutUCT,  // e_bUseCookie  = true
		m_eGmPutUCF,  // e_bUseCookie  = false
		m_eGmPutUMT,  // e_bUpdateMemo = true
		m_eGmPutUMF,  // e_bUpdateMemo = false
		m_eGmPutFRT,  // e_bFlashRoute = true
		m_eGmPutFRF,  // e_bFlashRoute = false
		m_eceGmPut,

		// Maximum and minimum values for messing coordinates
		m_eMinMess = -2,
		m_eMaxMess = 2,

		// Application defined messages
		WM_TASKBAR_ICON = WM_USER + 7,  // Taskbar icon notification
		WM_SEND_POI     = WM_APP,       // Sends POIs
		WM_RECEIVE_POI  = WM_SEND_POI,  // Receives POIs
		WM_DRAG_DROP,                   // Drag-to-drop
		WM_GPS_CHANGE,                  // GPS device has been changed

		// Return values for receiving POIs
		m_eRecvPoiUnused = 0,
		m_eRecvPoiBusy,
		m_eRecvPoiError,
		m_eRecvPoiOK = 7,

		// Opens/saves file/list from/to GPS device
		m_eGpsTimeout   = 2000,
		m_eGpsLoopIntl  = 16000,
		m_eGpsSleepIntl = 1000,
		m_ecbGpsAlloc   = 16 * 1024,
		m_ecbGpsBuffer  = 1024,
		m_ecbGpsPayload = MAX_STR * 3 + MAX_FNAME * FT_UTF8S3,

		// Response codes for GPS device
		m_eGpsOKRC       = 200,  // Success/File already exists
		m_eGpsCreatedRC  = 201,  // File is created/File does not exist
		m_eGpsUnknownRC  = 207,  // Unknown error
		m_eGpsNotFoundRC = 404,  // File not found
		m_eGpsSpaceRC    = 507,  // Disk space error
	};

public:
	PCTSTR            m_pszCmdLine;
	PTSTR             m_pszRecentFile, m_pszGmTempFile, m_pszGeTempFile;
	PTSTR             m_pszFileTitle, m_pszFileFilterR, m_pszFileFilterW, m_pszFileFilterB;
	BOOL              m_bFileDirty, m_bSetCaption, m_bSetReady, m_bAdjIEPane, m_bCustTbBtn;
	BOOL              m_bSortAscend, m_bAdjDistance, m_bMarquee, m_bLvBkgnd, m_bRPF, m_bMyBP;
	BOOL              m_bGmReady, m_bGmSelTimer, m_bGmSelItem, m_bMBtnDown, m_bLRKeyDown;
	BOOL              m_bBackup, m_bGpsDevice, m_bTomTomHome;
	BOOL              m_bSetBmp, m_bMoveBmp;
	int               m_iLvBaseItem, m_iInsertBP, m_iLvCol, m_ceRecentFile;
	int               m_cxClient, m_cyClient, m_cyToolbar, m_cyLvHeader, m_cyLvEdit;
	int               m_cyStatusBar, m_cxBmpPart, m_cxPoiPart, m_cxTimePart, m_cxyPartBorder;
	int               m_cePoiDupDel, m_ceDragDrop;
	UINT              m_nPoiDupSel, m_nPoiCbFmt, m_nPoiScCbFmt, m_nItnCbFmt;
	LONG              m_lRangeLngW, m_lRangeLngE;
	COLORREF          m_crTbBkgnd;
	double            m_dBaseLng, m_dBaseLat, m_dBaseRedLng, m_dBaseSinLat, m_dBaseCosLat;
	OPERPOIFILE       m_opf;
	FILESTRUCTTYPE    m_fst;
	GPSDUMP           m_gpsd;
	POIRECORD64       m_pr64;
	HBITMAP           m_hPoiBmp;
	HBRUSH            m_hMenuBarBkgnd;
	HDC               m_hSbDC;
	HFONT             m_hLvFont;
	HMENU             m_hSysMenu, m_hMainMenu, m_hFileMenu;
	HMENU             m_hPopMenu, m_hTbPopMenu, m_hHdPopMenu, m_hLvPopMenu;
	HTHREAD           m_hGpsDevThread, m_hNewVerThread;
	MENUHELP          m_mh[m_eceMenuHelp];
	CCheckDataDlg     m_jCheckDataDlg;
	CCheckPoiDlg      m_jCheckPoiDlg;
	CConvertFormatDlg m_jConvertFormatDlg;
	CCustTbDlg        m_jCustTbDlg;
	CFileDlg          m_jFileDlg;
	CFindReplDlg      m_jFindReplDlg;
	CFontDlg          m_jFontDlg;
	CImageList        m_jMenuItemIL, m_jToolbarIL, m_jToolbarHotIL, m_jTbChevronIL;
	CLvBkDlg          m_jLvBkDlg;
	CLvCombo          m_jLvCombo;
	CLvCtrl           m_jListView;
	CLvEdit           m_jLvEdit;
	CLvHeader         m_jLvHeader;
	COpenFavDlg       m_jOpenFavDlg;
	COptionDlg        m_jOptionDlg;
	CProgress         m_jProgress;
	CStatic           m_jPoiBmp;
	CSbCtrl           m_jStatusBar;
	CTbCtrl           m_jToolbar;
	CTranNameDlg      m_jTranNameDlg;

	// IE pane associated objects
	CIEPane                  m_jIEPane;
	IOleObject*              m_pOO;
	IWebBrowser2*            m_pWB2;
	IOleInPlaceActiveObject* m_pOIPAO;
	IDispatch*               m_pDsp;
	IHTMLDocument2*          m_pHD2;

public:
	static CTCHAR        m_szOv2Ext[4];
	static CTCHAR        m_szTxtExt[];
	static CTCHAR        m_szKmlExt[];
	static CTCHAR        m_szGpxExt[];
	static CTCHAR        m_szGpiExt[];
	static CTCHAR        m_szCsvExt[];
	static CTCHAR        m_szAscExt[];
	static CTCHAR        m_szPoiExt[];
	static CTCHAR        m_szPngExt[];
	static CTCHAR        m_szBmpExt[];
	static CTCHAR        m_szRpfExt[];
	static CTCHAR        m_szOggExt[];
	static CTCHAR        m_szVifExt[];
	static CTCHAR        m_szErrToken[];
	static CTCHAR        m_szCtrlToken[];
	static CTCHAR        m_szAmpEntity[];
	static CTCHAR        m_szLtEntity[];
	static CTCHAR        m_szGtEntity[];
	static CTCHAR        m_szQuotEntity[];
	static CTCHAR        m_szNbspEntity[];
	static CTCHAR        m_szLastNotifyEntry[];
	static CTCHAR        m_szOptionSection[];
	static CTCHAR        m_szShowMbEntry[];
	static CTCHAR        m_szShowTbEntry[];
	static CTCHAR        m_szLargeTbEntry[];
	static CTCHAR        m_szWrapTbEntry[];
	static CTCHAR        m_szTbBkgndEntry[];
	static CTCHAR        m_szTbButtonEntry[];
	static CTCHAR        m_szTbSpacingEntry[];
	static CTCHAR        m_szLngFirstEntry[];
	static CTCHAR        m_szDmsFormatEntry[];
	static CTCHAR        m_szCompassDirEntry[];
	static CTCHAR        m_szAutoColEntry[];
	static CTCHAR        m_szShowDistEntry[];
	static CTCHAR        m_szMetricUnitEntry[];
	static CTCHAR        m_szAdjDistEntry[];
	static CTCHAR        m_szRecentFileEntry[];
	static CTCHAR        m_szFileCountEntry[];
	static CTCHAR        m_szRecentListEntry[];
	static CTCHAR        m_szFadeWndEntry[];
	static CTCHAR        m_szShadeTableEntry[];
	static CTCHAR        m_szShowSbEntry[];
	static CTCHAR        m_szSignalMarkEntry[];
	static CTCHAR        m_szOptPoiEntry[];
	static CTCHAR        m_szCheckNameEntry[];
	static CTCHAR        m_szAutoBPEntry[];
	static CTCHAR        m_szOpenLastEntry[];
	static CTCHAR        m_szTranNameEntry[];
	static CTCHAR        m_szBackupFileEntry[];
	static CTCHAR        m_szBackupIntlEntry[];
	static CTCHAR        m_szGmLangEntry[];
	static CTCHAR        m_szGmCookieEntry[];
	static CTCHAR        m_szGmMemoEntry[];
	static CTCHAR        m_szGmRouteEntry[];
	static CTCHAR        m_szFontSection[];
	static CTCHAR        m_szFontNameEntry[];
	static CTCHAR        m_szFontWeightEntry[];
	static CTCHAR        m_szFontItalicEntry[];
	static CTCHAR        m_szFontColorEntry[];
	static CTCHAR        m_szFontBkgndEntry[];
	static CTCHAR        m_szFontUseBkEntry[];
	static CTCHAR        m_szIEPaneEntry[];
	static CBYTE         m_byUnicodeMark[];
	static CBYTE         m_byUtf8Mark[];
	static CPCTSTR       m_pszGPSF[GPSF_COUNT];
	static CTCHAR        m_pszGpsfSep[3];
	static CBYTE         m_byTfnCode[];
	static CPCTSTR       m_pszOptionEntry[];
	static CUINT         m_nOptionEntry[];
	static CPOIRECORD9DT m_pr9dt[];
	static TBBUTTON      m_tbb[];
	static VARIANT       m_vtEmpty;

public:
	CTomPoiEdr(PCTSTR pszCmdLine, PTSTR pszFileName, PTSTR pszFileTitle,
		PTSTR pszFileFilterR, PTSTR pszFileFilterW, PTSTR pszFileFilterB,
		PTSTR pszFindText, PTSTR pszReplaceText, PTSTR pszGmTempFile, PTSTR pszGeTempFile,
		PTSTR pszLvBkFName, PTSTR pszLvBkFTitle);
	void AddPoiRecord(int iLvItem);
	void AutoSizeLvCol(int iLvCol, BOOL bSetInfo);
	void BeginLvEdit(int iLvItem);
	LPARAM CalcPoiDistance(int iLvItem1, int iLvItem2);
	LPARAM CalcPoiDistance(int iLvItem);
	BOOL CheckSignalMark(PPOIRECORD64 pPR64, UINT nCheckSum);
	void CloseIEPane();
	void CombineCoordBlock(PLONG plDestCB, PCLONG plSrcCB);
	BOOL CopyPoiRecord();
	void CreateTbCtrl();
	UINT CustomDrawToolbar(PCNMCUSTOMDRAW pNMCD);
	void DecodeEntity(PTSTR pszData);
	void DeletePoiRecord(UINT nResId);
	void DisplayFileProgress(DWORD dwDividend, DWORD dwDivisor);
	int DisplayMessage(UINT nResId, UINT nType, LPARAM lParam);
	int DisplayMessage(UINT nResId, UINT nType);
	void DoAction(UINT nResId);
	int EncodeEntity(PTSTR pszData);
	int EncodeUrl(PTSTR pszUrl);
	void EnsureLvItemVisible(int iLvItem);
	int FindBasePoint();
	int FindPoiName(PCTSTR pszPoiName, int iLvItem);
	BOOL FindPoiName(PCTSTR pszPoiName);
	int FindPoiRecord(int iLvItem, BOOL bMarkAll);
	void GetGoogleMaps();
	void GetPurePathName(PCTSTR pszFileName, PTSTR pszBuffer);
	BYTE GetTomFileCode(WCHAR wChar);
	void GpsBroadcastChange(BOOL bGpsDevice);
	BOOL GpsCloseDialog(BOOL bPrevGpsDevice);
	void GpsCreateMarquee(UINT nResId);
	DWORD GpsGetFile(HINTERNET hInternet, BOOL bPoiPath,
		PCTSTR pszFileName, BOOL bOv2File, PTSTR pszTempFile);
	DWORD GpsGetFileList(HINTERNET hInternet, BOOL bPoiPath, PGPSFILELIST pGPSFL);
	BOOL GpsIsValidDump(HINTERNET hInternet, GPSDUMP gpsd);
	void GpsVerifyDump(HINTERNET hInternet);
	void InsertRecentFile(PCTSTR pszFileName, int iListNumber);
	void InsertRecentFile(PCTSTR pszFileName);
	void MoveLvItem(BOOL bMoveUp, ACTIONTYPE at);
	void OpenPoiFile(UINT nRecentId);
	void OperateFileName(OPERPOIFILE opf);
	BOOL OperatePoiFile(OPERPOIFILE opf);
	BOOL PeekUpdateMessage(BOOL bCheckEsc);
	BOOL PromptMessage();
	BOOL PtInIEPaneSep();
	BOOL PtInIEPaneSep(int xCursor, int yCursor);
	void PutGoogleMaps(int iPutCmd, int iLvItem);
	UINT ReadCpbRecord(PVOID pvFile, DWORD cbFile);
	void ReadCpbRecord(PCHAR pcStart, PCHAR pcEnd);
	void ReadCsvRecord(PWCHAR pwStart, PWCHAR pwEnd);
	UINT ReadCsvRecordA(PVOID pvFile, DWORD cbFile);
	void ReadCsvRecordU(PWCHAR pwFile, DWORD cwFile);
	UINT ReadGpiRecord(PVOID pvFile, DWORD cbFile);
	UINT ReadGpxRecord(PVOID pvFile, DWORD cbFile);
	UINT ReadKmlRecord(PVOID pvFile, DWORD cbFile);
	UINT ReadFavRecord(PVOID pvFile, DWORD cbFile);
	UINT ReadOv2Record(PVOID pvFile, DWORD cbFile);
	void ReadOv2Record2(PPOIRECORD2 pPR2, BOOL bVariety);
	void ReadOv2Record3(PPOIRECORD3 pPR3);
	void ReadOv2Record17(PPOIRECORD17 pPR17);
	void ReadOv2Record1D(PPOIRECORD1D pPR1D);
	void ReadOv2Record1E(PPOIRECORD1E pPR1E);
	BOOL ReadPoiFile();
	void ReadTxtRecord(PWCHAR pwStart, PWCHAR pwEnd);
	UINT ReadTxtRecordA(PVOID pvFile, DWORD cbFile);
	void ReadTxtRecordU(PWCHAR pwFile, DWORD cwFile);
	void ReleaseIEObject();
	void RemoveMenuItemAmp(PTSTR pszText);
	BOOL ReplacePoiRecord(int iLvItem, BOOL bReplAll);
	void ResizeWindow(BOOL bAllWnd);
	void SetBasePoint(int iLvItem, BOOL bAutoSize);
	BOOL SetCbData();
	void SetDistanceText(int iLvItem, BOOL bAutoSize);
	void SetDistanceText2(int iLvItem);
	void SetFileFilter(PCTSTR pszFileExt, BOOL bWriteExport);
	void SetLvFont();
	int ShowIEPane();
	void SortLvItems(int iLvCol, BOOL bAscend, ACTIONTYPE at);
	void SwapCoordCol();
	WCHAR TomChar2WinChar(WCHAR wTom);
	int TomFile2WinFile(PCTSTR pszFileName, PTSTR pszBuffer);
	void UpdateCoordData(BOOL bDmsFormat);
	void UpdateTbButton();
	void ValidateCoordCol(int iLvItem, int iLvCol, PTSTR pszText, BOOL bClearCol);
	void ValidateMemoCol(int iLvItem, PTSTR pszText);
	int ValidateNameCol(PTSTR pszText);
	void ValidateTelCol(int iLvItem, PTSTR pszText);
	WCHAR WinChar2TomChar(WCHAR wWin);
	int WinFile2TomFile(PCTSTR pszFileName, PTSTR pszBuffer);
	UINT WriteCpbRecord(HFILE hFile);
	UINT WriteCpbRecord(HFILE hFile, int iLvItem);
	UINT WriteCsvRecord(HFILE hFile);
	UINT WriteCsvRecord(HFILE hFile, int iLvItem);
	UINT WriteGpxRecord(HFILE hFile);
	UINT WriteKmlRecord(HFILE hFile, PINT pcePoint);
	UINT WriteOv2Record(HFILE hFile);
	UINT WriteOv2Record2(PVOID pvMemory, int iLvItem, BOOL bMemory);
	UINT WriteOv2Record1D(PVOID pvMemory, int iLvItem, BOOL bMemory);
	UINT WriteOv2Record1E(PVOID pvMemory, int iLvItem, BOOL bMemory);
	BOOL WritePoiFile();
	UINT WriteTxtRecord(HFILE hFile);
	UINT WriteTxtRecord(HFILE hFile, int iLvItem);

public:
	static BOOL GpsFindFile(PGPSFILE pGPSF, PCTSTR pszName);
	static void GpsFreeFileList(PGPSFILELIST pGPSFL);
	static HINTERNET GpsInternetOpen();

public:
	static int CALLBACK CompareLvItem(int iItem1, int iItem2, LPARAM lSortParam);
	static int CALLBACK CompareLvParam(LPARAM lParam1, LPARAM lParam2, LPARAM lSortParam);
	static UINT CALLBACK CustomDrawListView(PNMLVCUSTOMDRAW pNMLVCD);
	static DWORD CALLBACK GpsDeleteDumpFile(PVOID pvParam);
	static DWORD CALLBACK GpsDeletePoiFile(PVOID pvParam);
	static DWORD CALLBACK GpsDeviceChange(PVOID pvParam);
	static DWORD CALLBACK GpsPostPoiFile(PVOID pvParam);
	static DWORD CALLBACK NotifyNewVersion(PVOID pvParam);

protected:
	void OnActivate(UINT nState, HWND hOtherWnd, BOOL bMinimized);
	void OnFileNew();
	void OnFileOpen();
	void OnFileOpenFav();
	void OnFileOpenDevice();
	void OnFileDumpDevice();
	void OnFileSave();
	void OnFileSaveAs();
	void OnFileSaveDevice();
	void OnFileImport();
	void OnFileExport();
	void OnFileQuery();
	void OnFileTranslate();
	void OnFileConvert();
	void OnFileExit();
	void OnPoiUndo();
	void OnPoiRedo();
	void OnPoiAdd();
	void OnPoiInsert();
	void OnPoiCombineCol();
	void OnPoiDelete();
	void OnPoiClearCol();
	void OnPoiEdit();
	void OnPoiCut();
	void OnPoiCopy();
	void OnPoiPaste();
	void OnPoiCopyCoord();
	void OnPoiPasteCoord();
	void OnPoiQueryCoord();
	void OnPoiQueryAddr();
	void OnPoiFind();
	void OnPoiFindNext();
	void OnPoiReplace();
	void OnPoiReplaceNext();
	void OnPoiMessCoord();
	void OnPoiCheck();
	void OnPoiAddFav();
	void OnPoiAddItn();
	void OnPoiSelAll();
	void OnPoiToggleSel();
	void OnPosUp();
	void OnPosDown();
	void OnPosSetBP();
	void OnPosMyBP();
	void OnPosSortAscend();
	void OnPosSortDescend();
	void OnPosGoogleMap();
	void OnPosGoogleEarth();
	void OnToolsQGPSfix();
	void OnToolsShowMb();
	void OnToolsShowTb();
	void OnToolsLargeTb();
	void OnToolsWrapTb();
	void OnToolsTbBkgnd();
	void OnToolsCustTb();
	void OnToolsLvFont();
	void OnToolsLvBkgnd();
	void OnToolsShowSb();
	void OnToolsOption();
	void OnHelpAbout();
	void OnOpenCmdLine();
	void OnResizeWindow(LPARAM lParam);
	void OnSelectLvItem(LPARAM lParam);
	void OnLngFirst();
	void OnDmsFormat();
	void OnCompassDir();
	void OnAutoSizeCol();
	void OnClearCol();
	void OnClearAddr();
	void OnShowDistance();
	void OnMetricUnit();
	void OnAdjDistance();
	void OnBaseDistance();
	void OnSortAscend();
	void OnSortDescend();
	void OnAppKeyDown();
	void OnHelpMisc(UINT nItemId);
	void OnRecentFile(UINT nItemId);
	BOOL OnClose();
	int OnCreate(PCCREATESTRUCT pCS);
	void OnDestroy();
	UINT OnDeviceChange(UINT nEvent, PCVOID pvStruct);
	void OnDrawItem(PCDRAWITEMSTRUCT pDIS);
	void OnDropFiles(HDROP hDropFile);
	void OnGetMinMaxInfo(PMINMAXINFO pMMI);
	void OnInitMenu(HMENU hMenu);
	void OnLButtonDblClk(UINT nFlags, int xCursor, int yCursor);
	void OnLButtonDown(UINT nFlags, int xCursor, int yCursor);
	void OnLButtonUp(UINT nFlags, int xCursor, int yCursor);
	void OnMeasureItem(PMEASUREITEMSTRUCT pMIS);
	void OnMenuSelect(UINT nItemId, UINT nFlags, HMENU hMenu);
	void OnMouseMove(UINT nFlags, int xCursor, int yCursor);
	void OnHdnDividerDblClick(PCNMHEADER pNMH);
	void OnHdnEndTrack(PCNMHEADER pNMH);
	void OnHdnItemClick(PCNMHEADER pNMH);
	BOOL OnHdnTrack(PCNMHEADER pNMH);
	void OnLvnBeginDrag(PCNMLISTVIEW pNMLV);
	void OnLvnEndScroll(PCNMLVSCROLL pNMLVS);
	void OnLvnItemChanged(PCNMLISTVIEW pNMLV);
	BOOL OnLvnMarqueeBegin(PCNMLISTVIEW pNMLV);
	BOOL OnNmClick(PCNMHDR pNMH);
	UINT OnNmCustomDraw(PCNMCUSTOMDRAW pNMCD);
	BOOL OnNmDblClk(PCNMHDR pNMH);
	BOOL OnNmRClick(PCNMHDR pNMH);
	void OnSbnSimpleModeChange(PCNMHDR pNMH);
	void OnTbnBeginAdjust(PCNMHDR pNMH);
	void OnTbnDeletingButton(PCNMTOOLBAR pNMTB);
	BOOL OnTbnGetButtonInfo(PNMTOOLBAR pNMTB);
	void OnTbnGetInfoTip(PNMTBGETINFOTIP pNMTBGIT);
	int OnTbnInitCustomize(PCNMHDR pNMH);
	BOOL OnTbnQueryDelete(PCNMTOOLBAR pNMTB);
	BOOL OnTbnQueryInsert(PCNMTOOLBAR pNMTB);
	void OnTbnReset(PCNMHDR pNMH);
	void OnTbnToolbarChange(PCNMHDR pNMH);
	void OnPaint();
	BOOL OnSetCursor(HWND hCursorWnd, int iHitTest, UINT nMsg);
	void OnSetFocus(HWND hOldWnd);
	void OnSize(UINT nType, int cxClient, int cyClient);
	void OnSysColorChange();
	void OnSysCommand(UINT nItemId, LPARAM lParam);
	void OnTimer(UINT nTimerId);
	LRESULT OnTaskbarIcon(WPARAM wParam, LPARAM lParam);
	LRESULT OnReceivePoi(WPARAM wParam, LPARAM lParam);
	LRESULT OnDragDrop(WPARAM wParam, LPARAM lParam);
	LRESULT OnGpsChange(WPARAM wParam, LPARAM lParam);

	DECLARE_MESSAGE_MAP()
};

////////////////////////////////////////////////////////////////////////////////////////////////

AFXINLINE CTomPoiEdr::CTomPoiEdr(PCTSTR pszCmdLine, PTSTR pszFileName, PTSTR pszFileTitle,
	PTSTR pszFileFilterR, PTSTR pszFileFilterW, PTSTR pszFileFilterB,
	PTSTR pszFindText, PTSTR pszReplaceText, PTSTR pszGmTempFile, PTSTR pszGeTempFile,
	PTSTR pszLvBkFName, PTSTR pszLvBkFTitle) :
	m_jFileDlg(pszFileName, pszFileTitle), m_jFindReplDlg(pszFindText, pszReplaceText),
	m_jLvBkDlg(pszLvBkFName, pszLvBkFTitle)
{
	TRACE(_T("Entering CTomPoiEdr::CTomPoiEdr\n"));

	e_pTPE           = this;
	m_pszCmdLine     = pszCmdLine;
	m_pszGmTempFile  = pszGmTempFile;
	m_pszGeTempFile  = pszGeTempFile;
	m_pszFileTitle   = pszFileTitle;
	m_pszFileFilterR = pszFileFilterR;
	m_pszFileFilterW = pszFileFilterW;
	m_pszFileFilterB = pszFileFilterB;
}

// End of _CTOMPOIEDR_INCLUDED_
#endif
