////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
// Debug.hpp - Diagnostic Macros and Functions Declaration File                               //
//                                                                                            //
// Copyright (c) 1988-2013, Softop Corporation                                                //
// All rights reserved                                                                        //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

// Marks the file we have included
#ifndef _DEBUG_INCLUDED_
#define _DEBUG_INCLUDED_

// Framework header file should be included first
#ifndef _WINAFX_INCLUDED_
#error  Debug - Framework header file <WinAfx.hpp> should be included first.
#endif

// Included only once by the compiler in a build
#pragma once

////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG  // Debug version

// Allocates one or more objects
#define new new(_NORMAL_BLOCK, __FILE__, __LINE__)

// Displays the DLL loading and unloading states
void WINAPI DebugDllState(HMODULE hModule, DWORD dwReason);

// Reports an error message by a message box (used internally only)
BOOL WINAPI _DebugError(PCSTR pszFileName, int iLine, PCSTR pszMsg);

// Determines whether an expression is nonzero
#define ASSERT(tExp) \
	do \
	{ \
		if(!(tExp) && _DebugError(__FILE__, __LINE__, "ASSERT(" #tExp ")")) \
			_CrtDbgBreak(); \
	} \
	while(0)

// Outputs a warning message into the debug window (used internally only)
void WINAPI _DebugWarn(PCSTR pszFileName, int iLine, PCVOID pvMsg, BOOL bUnicode);
void WINAPI _DebugWarn(PCSTR pszFileName, int iLine, PCWSTR pszMsg);
void WINAPI _DebugWarn(PCSTR pszFileName, int iLine, PCSTR pszMsg);

// Determines whether an expression is nonzero
#define ASSERT_MSG(tExp, pszMsg) \
	do \
	{ \
		if(!(tExp)) \
			_DebugWarn(__FILE__, __LINE__, pszMsg); \
	} \
	while(0)

// Determines whether a window handle identifies an existing window
#define ASSERT_HWND(hWnd) \
	do \
	{ \
		if(!::IsWindow(hWnd) && _DebugError(__FILE__, __LINE__, "ASSERT_HWND(" #hWnd ")")) \
			_CrtDbgBreak(); \
	} \
	while(0)

// Determines whether a resulting value is succeeded
#define ASSERT_HRESULT(hResult) \
	do \
	{ \
		if(FAILED(hResult) && _DebugError(__FILE__, __LINE__, "ASSERT_HRESULT(" #hResult ")")) \
			_CrtDbgBreak(); \
	} \
	while(0)

// Determines whether a null-terminated string is valid (used internally only)
BOOL WINAPI _ASSERT_STR(PCWSTR pszText, PCSTR pszFileName, int iLine, PCSTR pszMsg);
BOOL WINAPI _ASSERT_STR(PCSTR pszText, PCSTR pszFileName, int iLine, PCSTR pszMsg);
BOOL WINAPI _ASSERT_NSTR(PCWSTR pszText, PCSTR pszFileName, int iLine, PCSTR pszMsg);
BOOL WINAPI _ASSERT_NSTR(PCSTR pszText, PCSTR pszFileName, int iLine, PCSTR pszMsg);
BOOL WINAPI _ASSERT_RSTR(PCWSTR pszText, PCSTR pszFileName, int iLine, PCSTR pszMsg);
BOOL WINAPI _ASSERT_RSTR(PCSTR pszText, PCSTR pszFileName, int iLine, PCSTR pszMsg);

// Determines whether a null-terminated string is valid
#define ASSERT_STR(pszText) \
	do \
	{ \
		if(_ASSERT_STR(pszText, __FILE__, __LINE__, "ASSERT_STR(" #pszText ")")) \
			_CrtDbgBreak(); \
	} \
	while(0)

// Determines whether a null-terminated string is valid (pointer can be NULL)
#define ASSERT_NSTR(pszText) \
	do \
	{ \
		if(_ASSERT_NSTR(pszText, __FILE__, __LINE__, "ASSERT_NSTR(" #pszText ")")) \
			_CrtDbgBreak(); \
	} \
	while(0)

// Determines whether a null-terminated string is valid (string must exactly exist)
#define ASSERT_RSTR(pszText) \
	do \
	{ \
		if(_ASSERT_RSTR(pszText, __FILE__, __LINE__, "ASSERT_RSTR(" #pszText ")")) \
			_CrtDbgBreak(); \
	} \
	while(0)

// Determines whether a resource identifier/pointer is valid (used internally only)
BOOL WINAPI _ASSERT_RES(PCWSTR pszResName, PCSTR pszFileName, int iLine, PCSTR pszMsg);
BOOL WINAPI _ASSERT_RES(PCSTR pszResName, PCSTR pszFileName, int iLine, PCSTR pszMsg);
BOOL WINAPI _ASSERT_RES(UINT nResId, PCSTR pszFileName, int iLine, PCSTR pszMsg);

// Determines whether a resource identifier/pointer is valid
#define ASSERT_RES(tRes) \
	do \
	{ \
		if(_ASSERT_RES(tRes, __FILE__, __LINE__, "ASSERT_RES(" #tRes ")")) \
			_CrtDbgBreak(); \
	} \
	while(0)

// Determines whether the calling process has read/write access to a range of memory
#define ASSERT_READ(pvMemory, cbSize) \
	do \
	{ \
		if(::IsBadReadPtr(pvMemory, cbSize) && \
			_DebugError(__FILE__, __LINE__, "ASSERT_READ(" #pvMemory ", " #cbSize ")")) \
			_CrtDbgBreak(); \
	} \
	while(0)
#define ASSERT_WRITE(pvMemory, cbSize) \
	do \
	{ \
		if(::IsBadWritePtr(pvMemory, cbSize) && \
			_DebugError(__FILE__, __LINE__, "ASSERT_WRITE(" #pvMemory ", " #cbSize ")")) \
			_CrtDbgBreak(); \
	} \
	while(0)

// Determines whether the calling process has read access to an address
#define ASSERT_PTR(pvAddress) \
	do \
	{ \
		if(::IsBadCodePtr((PROC)(pvAddress)) && \
			_DebugError(__FILE__, __LINE__, "ASSERT_PTR(" #pvAddress ")")) \
			_CrtDbgBreak(); \
	} \
	while(0)

// Statement cannot be reached
#define ASSERT_UNREACHABLE() \
	do \
	{ \
		if(_DebugError(__FILE__, __LINE__, "This statement is unreachable!")) \
			_CrtDbgBreak(); \
	} \
	while(0)

// Determines whether an expression result is equal/unequal to a value (used internally only)
int WINAPI _EQUAL(int iExpRet, int iOKValue, PCSTR pszFileName, int iLine, PCSTR pszMsg);
int WINAPI _UNEQUAL(int iExpRet, int iErrValue, PCSTR pszFileName, int iLine, PCSTR pszMsg);

// Determines whether an expression is nonzero
// Returns: Result of the expression
#define VERIFY(tExpRet) _UNEQUAL((int)(tExpRet), 0, __FILE__, __LINE__, "VERIFY(" #tExpRet ")")

// Determines whether a resulting value is succeeded (used internally only)
HRESULT WINAPI _VERIFY_HRESULT(HRESULT hResult, PCSTR pszFileName, int iLine, PCSTR pszMsg);

// Determines whether a resulting value is succeeded
// Returns: Original resulting value
#define VERIFY_HRESULT(hResult) \
	_VERIFY_HRESULT(hResult, __FILE__, __LINE__, "VERIFY_HRESULT(" #hResult ")")

// Determines whether an expression is equal to a value
// Returns: Result of the expression
#define EQUAL(tExpRet, tOKValue) \
	_EQUAL((int)(tExpRet), (int)(tOKValue), __FILE__, __LINE__, \
		"EQUAL(" #tExpRet ", " #tOKValue ")")

// Determines whether an expression is unequal to a value
// Returns: Result of the expression
#define UNEQUAL(tExpRet, tErrValue) \
	_UNEQUAL((int)(tExpRet), (int)(tErrValue), __FILE__, __LINE__, \
		"UNEQUAL(" #tExpRet ", " #tErrValue ")")

// Expression for debug version only
#define DEBUG_ONLY(tExp) tExp

// Enables/disables TRACE function
void WINAPI EnableTRACE();
void WINAPI DisableTRACE();

// Outputs a formatted message into the debug window
void WINAPIV TRACE(PCWSTR pszFormat, ...);
void WINAPIV TRACE(PCSTR pszFormat, ...);

// Outputs a formatted message into the debug window
void WINAPIV OUTPUT_STRING(PCWSTR pszFormat, ...);
void WINAPIV OUTPUT_STRING(PCSTR pszFormat, ...);

////////////////////////////////////////////////////////////////////////////////////////////////
#else  // Release version

// Removes the following macros
#define DebugDllState(hModule, dwReason) ((void)0)
#define ASSERT(tExp)                     ((void)0)
#define ASSERT_MSG(tExp, pszMsg)         ((void)0)
#define ASSERT_HWND(hWnd)                ((void)0)
#define ASSERT_HRESULT(hResult)          ((void)0)
#define ASSERT_STR(pszText)              ((void)0)
#define ASSERT_NSTR(pszText)             ((void)0)
#define ASSERT_RSTR(pszText)             ((void)0)
#define ASSERT_RES(tRes)                 ((void)0)
#define ASSERT_READ(pvMemory, cbSize)    ((void)0)
#define ASSERT_WRITE(pvMemory, cbSize)   ((void)0)
#define ASSERT_PTR(pvAddress)            ((void)0)
#define ASSERT_UNREACHABLE()             __assume(0)
#define VERIFY(tExpRet)                  (tExpRet)
#define VERIFY_HRESULT(hResult)          (hResult)
#define EQUAL(tExpRet, tOKValue)         (tExpRet)
#define UNEQUAL(tExpRet, tErrValue)      (tExpRet)
#define DEBUG_ONLY(tExp)

// Removes the following functions
#define EnableTRACE()  ((void)0)
#define DisableTRACE() ((void)0)
AFXINLINE void WINAPIV _TRACE(PCWSTR pszFormat, ...) { }
AFXINLINE void WINAPIV _TRACE(PCSTR pszFormat, ...) { }
#define TRACE         1 ? ((void)0) : _TRACE
#define OUTPUT_STRING 1 ? ((void)0) : _TRACE

// End of DEBUG and _DEBUG_INCLUDED_
#endif
#endif
