////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
// Debug.cpp - Diagnostic Macros and Functions Implementation File                            //
//                                                                                            //
// Copyright (c) 1988-2013, Softop Corporation                                                //
// All rights reserved                                                                        //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

#include <WinAfx.hpp>

// Entire file for debug version only
#ifdef DEBUG

////////////////////////////////////////////////////////////////////////////////////////////////
// Automatically executes when the application starts/exists (used internally only)

class _CDebug
{
public:
	~_CDebug();
};

_CDebug::~_CDebug()
{
	// Dumps all of the memory blocks when a memory leak has occurred
	if(_CrtDumpMemoryLeaks())
		::MessageBoxA(HWND_DESKTOP, "One or more memory leaks have occurred!",
			"WinAfx Static Library", MB_ICONWARNING | MB_OK);
}

INTERN _CDebug _i_jDebug;

////////////////////////////////////////////////////////////////////////////////////////////////
// Displays the DLL loading and unloading states

void WINAPI DebugDllState(HMODULE hModule, DWORD dwReason)
{
	if((dwReason != DLL_PROCESS_ATTACH) && (dwReason != DLL_PROCESS_DETACH))
		return;

	char szFileName[MAX_PATH];
	GetModuleFileNameA(hModule, szFileName, sizeof(szFileName));
	OutputDebugStringA((dwReason == DLL_PROCESS_ATTACH) ? "Loading '" : "Unloading '");
	OutputDebugStringA(szFileName);
	OutputDebugStringA("'\n");
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Reports an error message by a message box (used internally only)

BOOL WINAPI _DebugError(PCSTR pszFileName, int iLine, PCSTR pszMsg)
{
	// Removes WM_QUIT because if it is in the queue then the message box won't display
	BOOL bQuit;
	MSG  msg;
	bQuit = PeekMessage(&msg, NULL, WM_QUIT, WM_QUIT, PM_REMOVE);

	// Reports the message
	BOOL bBreak;
	bBreak = _CrtDbgReport(_CRT_ASSERT, strrchr(pszFileName, '\\') + 1, iLine, NULL, pszMsg);

	// Posts the WM_QUIT message back to queue, if necessary
	if(bQuit)
		PostQuitMessage(msg.wParam);

	// Whether the Retry button is pressed
	return bBreak;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Outputs a warning message into the debug window (used internally only)

void WINAPI _DebugWarn(PCSTR pszFileName, int iLine, PCVOID pvMsg, BOOL bUnicode)
{
	char szBuffer[16];
	OutputDebugStringA(strrchr(pszFileName, '\\') + 1);
	wsprintfA(szBuffer, "(%d): ", iLine);
	OutputDebugStringA(szBuffer);
	if(bUnicode)
		OutputDebugStringW((PCWSTR)pvMsg);
	else
		OutputDebugStringA((PCSTR)pvMsg);
	OutputDebugStringA("\n");
}

void WINAPI _DebugWarn(PCSTR pszFileName, int iLine, PCWSTR pszMsg)
{
	_DebugWarn(pszFileName, iLine, pszMsg, TRUE);
}
void WINAPI _DebugWarn(PCSTR pszFileName, int iLine, PCSTR pszMsg)
{
	_DebugWarn(pszFileName, iLine, pszMsg, FALSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Determines whether a null-terminated string is valid (used internally only)

enum _STRTYPE { _ST_NORMAL, _ST_NULL, _ST_REAL };
BOOL WINAPI _ASSERT_STR(PCVOID pvText, PCSTR pszFileName, int iLine, PCSTR pszMsg,
	_STRTYPE st, BOOL bUnicode)
{
	if((st == _ST_NULL) && !pvText)
		return FALSE;
	BOOL bBadStr;
	if(bUnicode)
		bBadStr = IsBadStringPtrW((PCWSTR)pvText, MAX_UINT);
	else
		bBadStr = IsBadStringPtrA((PCSTR)pvText, MAX_UINT);
	if(!bBadStr && ((st != _ST_REAL) || (bUnicode ? *(PCWCHAR)pvText : *(PCCHAR)pvText)))
		return FALSE;
	return _DebugError(pszFileName, iLine, pszMsg);
}

BOOL WINAPI _ASSERT_STR(PCWSTR pszText, PCSTR pszFileName, int iLine, PCSTR pszMsg)
{
	return _ASSERT_STR(pszText, pszFileName, iLine, pszMsg, _ST_NORMAL, TRUE);
}
BOOL WINAPI _ASSERT_STR(PCSTR pszText, PCSTR pszFileName, int iLine, PCSTR pszMsg)
{
	return _ASSERT_STR(pszText, pszFileName, iLine, pszMsg, _ST_NORMAL, FALSE);
}
BOOL WINAPI _ASSERT_NSTR(PCWSTR pszText, PCSTR pszFileName, int iLine, PCSTR pszMsg)
{
	return _ASSERT_STR(pszText, pszFileName, iLine, pszMsg, _ST_NULL, TRUE);
}
BOOL WINAPI _ASSERT_NSTR(PCSTR pszText, PCSTR pszFileName, int iLine, PCSTR pszMsg)
{
	return _ASSERT_STR(pszText, pszFileName, iLine, pszMsg, _ST_NULL, FALSE);
}
BOOL WINAPI _ASSERT_RSTR(PCWSTR pszText, PCSTR pszFileName, int iLine, PCSTR pszMsg)
{
	return _ASSERT_STR(pszText, pszFileName, iLine, pszMsg, _ST_REAL, TRUE);
}
BOOL WINAPI _ASSERT_RSTR(PCSTR pszText, PCSTR pszFileName, int iLine, PCSTR pszMsg)
{
	return _ASSERT_STR(pszText, pszFileName, iLine, pszMsg, _ST_REAL, FALSE);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Determines whether a resource identifier/pointer is valid (used internally only)

enum _RESTYPE { _RT_STRW, _RT_STRA, _RT_ID };
BOOL WINAPI _ASSERT_RES(PCVOID pvRes, PCSTR pszFileName, int iLine, PCSTR pszMsg, _RESTYPE rt)
{
	if(pvRes)
	{
		if((UINT)pvRes <= MAX_RESID)
			return FALSE;
		if(rt != _RT_ID)
		{
			BOOL bBadStr;
			if(rt == _RT_STRW)
				bBadStr = IsBadStringPtrW((PCWSTR)pvRes, MAX_UINT);
			else
				bBadStr = IsBadStringPtrA((PCSTR)pvRes, MAX_UINT);
			if(!bBadStr && ((rt == _RT_STRW) ? *(PCWCHAR)pvRes : *(PCCHAR)pvRes))
				return FALSE;
		}
	}
	return _DebugError(pszFileName, iLine, pszMsg);
}

BOOL WINAPI _ASSERT_RES(PCWSTR pszResName, PCSTR pszFileName, int iLine, PCSTR pszMsg)
{
	return _ASSERT_RES(pszResName, pszFileName, iLine, pszMsg, _RT_STRW);
}
BOOL WINAPI _ASSERT_RES(PCSTR pszResName, PCSTR pszFileName, int iLine, PCSTR pszMsg)
{
	return _ASSERT_RES(pszResName, pszFileName, iLine, pszMsg, _RT_STRA);
}
BOOL WINAPI _ASSERT_RES(UINT nResId, PCSTR pszFileName, int iLine, PCSTR pszMsg)
{
	return _ASSERT_RES((PCVOID)nResId, pszFileName, iLine, pszMsg, _RT_ID);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Determines whether an expression result is equal/unequal to a value (used internally only)

int WINAPI _EQUAL(int iExpRet, int iOKValue, PCSTR pszFileName, int iLine, PCSTR pszMsg)
{
	if((iExpRet != iOKValue) && _DebugError(pszFileName, iLine, pszMsg))
		_CrtDbgBreak();
	return iExpRet;
}
int WINAPI _UNEQUAL(int iExpRet, int iErrValue, PCSTR pszFileName, int iLine, PCSTR pszMsg)
{
	if((iExpRet == iErrValue) && _DebugError(pszFileName, iLine, pszMsg))
		_CrtDbgBreak();
	return iExpRet;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Determines whether a resulting value is succeeded (used internally only)

HRESULT WINAPI _VERIFY_HRESULT(HRESULT hResult, PCSTR pszFileName, int iLine, PCSTR pszMsg)
{
	if(FAILED(hResult) && _DebugError(pszFileName, iLine, pszMsg))
		_CrtDbgBreak();
	return hResult;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Enables/disables TRACE function

INTERN BOOL _i_bTrace;

void WINAPI EnableTRACE()
{
	_i_bTrace = TRUE;
}

void WINAPI DisableTRACE()
{
	_i_bTrace = FALSE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Outputs a formatted message into the debug window

void WINAPIV TRACE(PCWSTR pszFormat, ...)
{
	if(!_i_bTrace)
		return;

	WCHAR   szMsg[1024];
	va_list pArgList;
	va_start(pArgList, pszFormat);
	ASSERT(_vsnwprintf(szMsg, countof(szMsg), pszFormat, pArgList) >= 0);
	OutputDebugStringW(szMsg);
}
void WINAPIV TRACE(PCSTR pszFormat, ...)
{
	if(!_i_bTrace)
		return;

	char    szMsg[1024];
	va_list pArgList;
	va_start(pArgList, pszFormat);
	ASSERT(_vsnprintf(szMsg, sizeof(szMsg), pszFormat, pArgList) >= 0);
	OutputDebugStringA(szMsg);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Outputs a formatted message into the debug window

void WINAPIV OUTPUT_STRING(PCWSTR pszFormat, ...)
{
	WCHAR   szMsg[1024];
	va_list pArgList;
	va_start(pArgList, pszFormat);
	ASSERT(_vsnwprintf(szMsg, countof(szMsg), pszFormat, pArgList) >= 0);
	OutputDebugStringW(szMsg);
}
void WINAPIV OUTPUT_STRING(PCSTR pszFormat, ...)
{
	char    szMsg[1024];
	va_list pArgList;
	va_start(pArgList, pszFormat);
	ASSERT(_vsnprintf(szMsg, sizeof(szMsg), pszFormat, pArgList) >= 0);
	OutputDebugStringA(szMsg);
}

// End of DEBUG
#endif
